C
C     REFMAC refinement program
C     Copyright (C) 2000-2010 Garib Murshudov
C
C     This code is distributed under the terms and conditions of the
C     CCP4 licence agreement as `Part 2' (Annex 2) software.
C     A copy of the CCP4 licence can be obtained by writing to the
C     CCP4 Secretary, Darsesbury Laboratory, Warrington WA4 4AD, UK.
C
      PROGRAM refmac5
      use bvalue_routines
C------------------------------------------------------------------------
C----Remarks on program organization
C     Program for refinement macromolecular structures. Current 
C     ersion of program works as Hendickson-Konnert restrained refinement
c     with or without restraints and with FFT
C
C     Rigid body refinement
c    
c----CCPFYP
c----MTZINI        subroutines makes possible to work with CCP4
c
c
c----DEFLTS        defines default values of parameters
c
c----RCARD         reads necessary parametrs for refinement
c
c----HKREF         refines macromolecular structures using output from
c----              PROTIN. This subroutine works as HK + Agarwal program
c
C------------------------------------------------------------------------
      INCLUDE 'atom_com.fh'
      INCLUDE 'models.fh'
      INCLUDE 'refi_flags.fh'
      INCLUDE 'celsym.fh'
      INCLUDE 'celsym_aniso.fh'
      INCLUDE 'cif_incl.fh'
      INCLUDE 'monitor.fh'
      INCLUDE 'vitals.fh'
      INCLUDE 'expcost.fh'
      INCLUDE 'const.fh'
      include 'anom.fh'
      INCLUDE 'makecif_version.fh'
C
      integer ierr,nmodel,im
c
      COMMON /FILES / ISYSR,ISYSW,IFOFC,ISCRF,IATMR,IDISK,JDISK,NEWFIL
     .               ,INXYZ,IOUTX,MTZIN
C
C----Initialize ccp4 like file organization
      CALL CCP4_PROG_VERSION(R_VERSNO,0)
      CALL CCPFYP
      CALL MTZINI
      CALL DEFLTS
c-----------------------------------------------------
c
c----Open system input and output
c
c----------------------------------------------------
      CALL CCPRCS(ISYSW,'Refmac_'//R_VERSNO,
     & '$Date: '//R_VERSDATE//' $')
      write (isysw,11)
 11   format (
     +     ' $TEXT:Reference1: $$ comment $$ '/2x,
     +     ' "Refinement of Macromolecular Structures by the ',
     +     ' Maximum-Likelihood Method:"'/2x,
     +     ' G.N. Murshudov, A.A.Vagin and E.J.Dodson,(1997)'/2x,
     +     ' Acta Crystallogr. D53, 240-255'/2x,
     +     ' EU  Validation contract: BIO2CT-92-0524',//,' $$',/,
     +     ' $SUMMARY :Reference1:  $$ Refmac: $$'/
     +     ' :TEXT:Reference1: $$'/)
C
c----Initialise for harvesting and read cards and instructions
      CALL rharvests(0)
      CALL RCARD
c---------------------------------------------------
c----Read coordinate and reflection files if necessary
c---------------------------------------------------
      IF(MODE.EQ.'RIGI') REFID = 'UNRE'

      IF(CRDUSE) CALL OPPRO(nmodel)
      IF(REFUSE) CALL REFPRE
c
C---------------------------------------------------
C---General preperation. 
C---Calc orth. deorth matrices and so on
C--------------------------------------------------
      CALL GENPRE
C
C---If all atoms should be anisotropic then convert those which are not
C---anisotropic yet
      if(itemp.eq.1.or.itemp.eq.0) then
         call allanisous2b(nmodel)
      elseif(ITEMP.EQ.2) then 
         CALL ALLBS2ANISOU(nmodel)
      elseif(itemp.eq.3) then
         call add_mixed_bvalues(nmodel)
      else
         call ccperr(1,'Unknown type for B value refinement')
      endif
      CALL CHECK_U_VALUES(nmodel)

c
c----Print out some more useful info
      call print_prerefine_info
c
C---Begin actions
      IFLAGB = 0
      IFLAGE = 1
      CALL CCPTIM(IFLAGE,CPU,ELAPS)
      CALL CCPTIM(IFLAGB,CPU,ELAPS)
 6018 FORMAT ('  Time in seconds: CPU = ',F12.2,/,
     +        '             Elapsed =   ',F12.2)
C
      IF(MODE.EQ.'HKRF') THEN
         if(rand_ref_flag) then
            if(gibbs_sample_flag) then
               call gm_mc_coefs(nmodel)
            else
               call randomise_and_refine(nmodel)
            endif
         elseif(uval_estim_flag) then
            call gm_mc_uval(nmodel)
         else
            CALL refine_individual(nmodel)
c            call calc_derivs(nmodel)
         endif
c         call tls_and_individual
      ELSEIF(MODE.EQ.'CELR') THEN
cd        CALL CELLREF(NAME)
      ELSEIF(MODE.EQ.'RIGI') THEN
        CALL RIGID_BODY
      ELSEIF(MODE.EQ.'TLSR') THEN
        CALL TLS_REFINE
      ELSEIF(MODE.EQ.'NEWE') THEN
        CALL NEWENTRY_IDEALISE
      elseif(mode.eq.'OCCU'.OR.REFID.EQ.'OCCU') then
         call occup_only_refine(nmodel)
cd      ELSEIF(MODE.EQ.'INTE') THEN
cd        CALL INTERNAL_REFINE
c
c     ELSEIF(MODE.EQ.'CONS') THEN
C        CALL  CONSTR
C     ELSEIF(MODE.EQ.'FULL') THEN
C        CALL FULLM
C     ELSEIF(MODE.EQ.'OMT1') THEN
C        CALL OMIT1
C     ELSEIF(MODE.EQ.'OMT2') THEN
C        CALL OMIT2
C     ELSEIF(MODE.EQ.'BLCK') THEN
C        CALL BLOCKR
C     ELSEIF(MODE.EQ.'SFCL') THEN
C        CALL SFCALC
C     ELSEIF(MODE.EQ.'MAPC') THEN
C        CALL MAPCALC
C     ELSEIF(MODE.EQ.'AUTO') THEN
C        CALL AUTOREF
      ELSE
        CALL ERRWRT(1,' Other modes are not active')
      ENDIF
C
C---Another action?
C     IF(ICONTINUE.EQ.1) GOTO 1
C                ************************
      do im=1,nmodel
        CALL WRITE_ATOMS_REFMAC(im)
      enddo
c
      if(refmac_tests.eq.'DBAS') then
         mode = 'OCCU'
         refid = 'UNRE'
         call occup_only_refine(nmodel)
      endif
c
      IFLAGB = 0
      IFLAGE = 1
      CALL CCPTIM(IFLAGE,CPU,ELAPS)
      WRITE (ISYSW,FMT=6018) CPU,ELAPS
      CALL CCPTIM(IFLAGB,CPU,ELAPS)
C
C---Write harvesting data
      CALL WRITE_VS_CYCLE
      CALL rharvests(1)
      ierr = 0
      call write_xml(ierr,'normal termination')

      CALL REFMAC_CLEAN_UP_FILES
      CALL CCPERR(0,'End of Refmac_'//R_VERSNO)
      END

      SUBROUTINE REFMAC_CLEAN_UP_FILES
      use agreem
      use restr_files
      IMPLICIT NONE
C
C---Removes intermadiate files used
      INCLUDE 'makecif.fh'
      LOGICAL LEXISTS
      INTEGER IFAIL,IUNIT,ISYSR,ISYSW,IFOFC,ISCRF,IATMR,IDISK,JDISK,
     &        NEWFIL,INXYZ,IOUTX,MTZIN,LL
      COMMON /FILES / ISYSR,ISYSW,IFOFC,ISCRF,IATMR,IDISK,JDISK,NEWFIL
     .               ,INXYZ,IOUTX,MTZIN
C
      IF(STRUCT_FILE(1:1).NE.' ') THEN
      INQUIRE (FILE=STRUCT_FILE,EXIST=LEXISTS)
      IF(LEXISTS) THEN
         call open_unform_file(iunit,struct_file,ifail)
        CLOSE(IUNIT,STATUS='DELETE')
      ENDIF
      ENDIF
C
      IF(RESTRAINT_FILE(1:1).NE.' ') THEN
      INQUIRE (FILE=RESTRAINT_FILE,EXIST=LEXISTS)
      IF(LEXISTS.AND.RESTRAINT_FILE(1:1).NE.' ') THEN
         call open_unform_file(iunit,restraint_file,ifail)
        CLOSE(IUNIT,STATUS='DELETE')
      ENDIF
      ENDIF
C
      IF(COORD_FILE(1:1).NE.' ') THEN
      INQUIRE (FILE=COORD_FILE,EXIST=LEXISTS)
      IF(LEXISTS.AND.COORD_FILE(1:1).NE.' ') THEN
         call open_unform_file(iunit,coord_file,ifail)
        CLOSE(IUNIT,STATUS='DELETE')
      ENDIF
      ENDIF
C
      IF(BOND_FILE(1:1).NE.' ') THEN
      INQUIRE (FILE=BOND_FILE,EXIST=LEXISTS)
      IF(LEXISTS.AND.BOND_FILE(1:1).NE.' ') THEN
         call open_unform_file(iunit,bond_file,ifail)
        CLOSE(IUNIT,STATUS='DELETE')
      ENDIF
      ENDIF
      IF(len_Trim(BOND_FILE_EXTRA).GT.0) THEN
      INQUIRE (FILE=BOND_FILE_EXTRA,EXIST=LEXISTS)
      IF(LEXISTS.AND.BOND_FILE_EXTRA(1:1).NE.' ') THEN
         call open_unform_file(iunit,bond_file_extra,ifail)
        CLOSE(IUNIT,STATUS='DELETE')
      ENDIF
      ENDIF
      
      IF(len_trim(ANGLE_FILE).GT.0) THEN
      INQUIRE (FILE=ANGLE_FILE,EXIST=LEXISTS)
      IF(LEXISTS.AND.ANGLE_FILE(1:1).NE.' ') THEN
         call open_unform_file(iunit,angle_file,ifail)
        CLOSE(IUNIT,STATUS='DELETE')
      ENDIF
      ENDIF
      IF(len_trim(ANGLE_FILE_EXTRA).GT.0) THEN
      INQUIRE (FILE=ANGLE_FILE_EXTRA,EXIST=LEXISTS)
      IF(LEXISTS.AND.ANGLE_FILE_EXTRA(1:1).NE.' ') THEN
         call open_unform_file(iunit,angle_file_extra,ifail)
        CLOSE(IUNIT,STATUS='DELETE')
      ENDIF
      ENDIF      
      IF(VDW_FILE(1:1).NE.' ') THEN
      INQUIRE (FILE=VDW_FILE,EXIST=LEXISTS)
      IF(LEXISTS.AND.VDW_FILE(1:1).NE.' ') THEN
         call open_unform_file(iunit,vdw_file,ifail)
        CLOSE(IUNIT,STATUS='DELETE')
      ENDIF
      ENDIF
      IF(VDW_FILE_0(1:1).NE.' '.AND.len_Trim(VDW_FILE_0).GT.0) THEN
      INQUIRE (FILE=VDW_FILE_0,EXIST=LEXISTS)
      IF(LEXISTS.AND.VDW_FILE_0(1:1).NE.' ') THEN
         call open_unform_file(iunit,vdw_file_0,ifail)
        CLOSE(IUNIT,STATUS='DELETE')
      ENDIF
      ENDIF
      IF(CHIR_FILE(1:1).NE.' ') THEN
      INQUIRE (FILE=CHIR_FILE,EXIST=LEXISTS)
      IF(LEXISTS.AND.CHIR_FILE(1:1).NE.' ') THEN
         call open_unform_file(iunit,chir_file,ifail)
        CLOSE(IUNIT,STATUS='DELETE')
      ENDIF      
      ENDIF

      IF(TORS_FILE(1:1).NE.' ') THEN
      INQUIRE (FILE=TORS_FILE,EXIST=LEXISTS)
      IF(LEXISTS.AND.TORS_FILE(1:1).NE.' ') THEN
         call open_unform_file(iunit,tors_file,ifail)
        CLOSE(IUNIT,STATUS='DELETE')
      ENDIF
      ENDIF
 
      IF(NCSR_FILE_NAME(1:1).NE.' ') THEN
      INQUIRE (FILE=NCSR_FILE_NAME,EXIST=LEXISTS)
      IF(LEXISTS.AND.NCSR_FILE_NAME(1:1).NE.' ') THEN
         call open_unform_file(iunit,ncsr_file_name,ifail)
        CLOSE(IUNIT,STATUS='DELETE')
      ENDIF
      ENDIF
C
      IF(NCSC_FILE_NAME(1:1).NE.' ') THEN
      INQUIRE (FILE=NCSC_FILE_NAME,EXIST=LEXISTS)
      IF(LEXISTS.AND.NCSC_FILE_NAME(1:1).NE.' ') THEN
         call open_unform_file(iunit,ncsc_file_name,ifail)
        CLOSE(IUNIT,STATUS='DELETE')
      ENDIF
      ENDIF
C
      IF(PLANE_FILE(1:1).NE.' ') THEN
      INQUIRE (FILE=PLANE_FILE,EXIST=LEXISTS)
      IF(LEXISTS.AND.PLANE_FILE(1:1).NE.' ') THEN
         call open_unform_file(iunit,plane_file,ifail)
        CLOSE(IUNIT,STATUS='DELETE')
      ENDIF
      ENDIF
      if(keywords_file(1:1).ne.' ') then
         INQUIRE (FILE=keywords_file,EXIST=LEXISTS)
         IF(LEXISTS.AND.keywords_file(1:1).NE.' ') THEN
            call open_form_file(iunit,keywords_file,ifail)
            CLOSE(IUNIT,STATUS='DELETE')
         ENDIF
      ENDIF 
      if(fomap_file(1:1).ne.' ') then
         inquire(file=fomap_file,exist=lexists)
         if(lexists) then
            call open_unform_file(iunit,fomap_file,ifail)
            close(iunit,status='delete')
         endif
      endif
      if(len_trim(refl_file).gt.0) then
         inquire(file=refl_File,exist=lexists)
         if(lexists) then
            call open_unform_file(iunit,refl_file,ifail)
            close(iunit,status='delete')
         endif
      endif
      if(len_trim(refl_asym).gt.0) then
         inquire(file=refl_asym,exist=lexists)
         if(lexists) then
            call open_unform_file(iunit,refl_asym,ifail)
            close(iunit,status='delete')
         endif
      endif
      if(len_trim(fobs_appr_file).gt.0) then
         inquire(file=fobs_appr_file,exist=lexists)
         if(lexists) then
            call open_unform_file(iunit,fobs_appr_file,ifail)
            close(iunit,status='delete')
         endif
      endif
C
      if(len_trim(input_orig_data).gt.0) then
         inquire(file=input_orig_data,exist=lexists)
         if(lexists) then
            call open_unform_file(iunit,input_orig_data,ifail)
            close(iunit,status='delete')
         endif
      endif
      if(len_trim(interval_file).gt.0) then
         inquire(file=interval_file,exist=lexists)
         if(lexists) then
            call open_unform_file(iunit,interval_file,ifail)
            close(iunit,status='delete')
         endif
      endif
      if(harmonic_file(1:1).ne.' ') then
         inquire(file=harmonic_file,exist=lexists)
         if(lexists) then
            call open_unform_file(iunit,harmonic_file,ifail)
            close(iunit,status='delete')
         endif
      endif
      if(len_trim(all_pairs_file).gt.0) then
         inquire(file=all_pairs_file,exist=lexists)
         if(lexists) then
            call open_unform_file(iunit,all_pairs_file,ifail)
            close(iunit,status='delete')
         endif
      endif
      if(len_trim(all_contacts_file).gt.0) then
         inquire(file=all_contacts_file,exist=lexists)
         if(lexists) then
            call open_unform_file(iunit,all_contacts_file,ifail)
            close(iunit,status='delete')
         endif
      endif
      if(len_trim(ncs_simil_file).gt.0) then
         inquire(file=ncs_simil_file,exist=lexists)
         if(lexists) then
            call open_unform_file(iunit,ncs_simil_file,ifail)
            close(iunit,status='delete')
         endif
      endif
      IF(ISCRF.GT.0) CLOSE(ISCRF,STATUS='DELETE')
      IF(ISCRF0.GT.0) CLOSE(ISCRF0,STATUS='DELETE')
      IF(IFSCR_SAVE.GT.0) CLOSE(IFSCR_SAVE,STATUS='DELETE')

      RETURN
      END
C
      SUBROUTINE WRITE_VS_CYCLE
      use rharvest
C
C---This routine writes some info vs cycle into ouput log file
C---things like R factor free etc
      IMPLICIT NONE
C
      INTEGER ICC,ic0
      CHARACTER LINE*94
C
      CALL HEADER('Things for loggraph, R factor and others vs cycle')
      CALL ERRWRT(-1,' ')
      CALL ERRWRT(-1,
     &     '$TABLE: Rfactor analysis, stats vs cycle  :')
      CALL ERRWRT(-1,'$GRAPHS:<Rfactor> vs cycle :N:1,2,3:')
      CALL ERRWRT(-1,':FOM vs cycle :N:1,4:')
C     >IJT
      CALL ERRWRT(-1,':-LL vs cycle :N:1,5:')
      CALL ERRWRT(-1,':-LLfree vs cycle :N:1,6:')
      CALL ERRWRT(-1,':Geometry vs cycle:N:1,7,8,9,10,11:')
C<IJT
      CALL ERRWRT(-1,'$$')
      CALL ERRWRT(-1,
C>IJT
     &     '    Ncyc    Rfact    Rfree     FOM      -LL     -LLfree'//
     &     '  rmsBOND  zBOND rmsANGL  zANGL rmsCHIRAL $$')
C<IJT
      CALL ERRWRT(-1,'$$')
C
      DO    ICC = 1,NCYCLE_OVERALL
         LINE = ' '
C>IJT
         WRITE(LINE(:8),'(I8)')ICC-1
         IF(RFACTOR_VS_CYCLE(ICC).GT.0.0) THEN
            WRITE(LINE(9:17),'(1X,F8.4)')RFACTOR_VS_CYCLE(ICC)
         ELSE
            LINE(9:17) = '   0.0'
         ENDIF
         IF(RFREE_VS_CYCLE(ICC).GT.0.0) THEN
            WRITE(LINE(18:26),'(1X,F8.4)')RFREE_VS_CYCLE(ICC)
         ELSE
            LINE(18:26) = '   0.0'
         ENDIF
         IF(FOM_VS_CYCLE(ICC).GT.0.0) THEN
            WRITE(LINE(27:34),'(1X,F7.3)')FOM_VS_CYCLE(ICC)
         ELSE
            LINE(27:34) = '   0.0'
         ENDIF
         WRITE(LINE(35:45),'(1X,F10.0)')LLG_VS_CYCLE(ICC)
         WRITE(LINE(46:55),'(1X,F9.1)')LLGFREE_VS_CYCLE(ICC)
         IF(BOND_VS_CYCLE(ICC).GT.0.0) THEN
            WRITE(LINE(56:64),'(1X,F8.4)')BOND_VS_CYCLE(ICC)
            WRITE(LINE(65:71),'(1X,F6.3)')ZBOND_VS_CYCLE(ICC)
         ELSE
            LINE(56:71) = '   0.0      0.0'
         ENDIF
         IF(ANGLE_VS_CYCLE(ICC).GT.0.0) THEN
            WRITE(LINE(72:79),'(1X,F7.3)')ANGLE_VS_CYCLE(ICC)
            WRITE(LINE(80:86),'(1X,F6.3)')ZANGLE_VS_CYCLE(ICC)
         ELSE
            LINE(72:86) = '   0.0     0.0'
         ENDIF
         IF(CHIR_VS_CYCLE(ICC).GT.0.0) THEN
            WRITE(LINE(87:),'(1X,F7.3)')CHIR_VS_CYCLE(ICC)
         ELSE
            LINE(87:) = '   0.0'
C<IJT
         endif
         
         CALL ERRWRT(-1,LINE)
      ENDDO
      CALL ERRWRT(-1,' $$ ')
c
c---  Write things for baubles
      write(*,*)'$TEXT:Result: $$ Final results $$'
      if(rfactor_vs_cycle(1).gt.0.0) then
         write(*,'(a20,2(2x,a7))')' ','Initial', 'Final'
         write(*,'(a20,2(2x,f7.4))')'R factor ',rfactor_vs_cycle(1),
     &           rfactor_vs_cycle(ncycle_overall)

         if(rfree_vs_cycle(1).gt.0.0) then
            write(*,'(a20,2(2x,f7.4))')'R free ',rfree_vs_cycle(1),
     &           rfree_vs_cycle(ncycle_overall)
         endif
         ic0 = 0
         do icc=1,ncycle_overall
            if(bond_vs_cycle(icc).ge.0.0) then
               ic0 = icc
               exit
            endif
         enddo
         if(ic0.gt.0) then
            if(bond_vs_cycle(ncycle_overall).gt.0.) then
               write(*,'(a20,2(2x,f7.4))')
     &              'Rms BondLength ',bond_vs_cycle(ic0),
     &              bond_vs_cycle(ncycle_overall)
            endif
            if(angle_vs_cycle(ncycle_overall).gt.0.) then
               write(*,'(a20,2(2x,f7.4))')
     &              'Rms BondAngle ',angle_vs_cycle(ic0),
     &              angle_vs_cycle(ncycle_overall)
            endif
            if(chir_vs_cycle(ncycle_overall).gt.0.) then
               write(*,'(a20,2(2x,f7.4))')
     &              'Rms ChirVolume ',chir_vs_cycle(ic0),
     &              chir_vs_cycle(ncycle_overall)
            endif
         endif
      endif
      write(*,*)'$$'
      RETURN
      END
c
      subroutine  calc_bvalue_stats(nch_this,number_gr,bstats_gr)  
      use rharvest
      implicit none
      include 'atom_com.fh'
      include 'models.fh'
      include 'refi_flags.fh'
c
      integer nch_this
      integer number_gr(3,nch_this)
      real bstats_gr(3,2,nch_this)
c
c---Calculate overall B values for all atoms, main and side chains      
      integer ich
      integer i,j,k,l,ir,nside,nmain,nall
      real occ,occ2
      real bloc,bloc2,pi,eightpisq
      character side_or_main*1
c
      do i=1,3
         do l=1,n_group+1
            number_gr(i,l) = 0
            do k=1,2
               bstats_gr(i,k,l) = 0.0
            enddo
         enddo
      enddo
c
      pi = 4.0*atan2(1.0,1.0)
      eightpisq = 8.0*pi**2
      do i=1,n_atom
         ir = i_resid(i)
         ich = i_chain(ir)
         if(occup_mod(i,1).gt.0.0) then
            if(atom_ref_mod_flag(i,1).gt.0) then
               if(u_aniso_mod(2,i,1).le.0.0) then
                  bloc = u_aniso_mod(1,i,1)
               else
                  bloc = sum(u_aniso_mod(1:3,i,1))/3.0
               endif
               bloc = eightpisq*bloc
               bloc2 = bloc**2
               call check_side_or_main(atm_name(i),side_or_main)
               ir = i_resid(i)
               if(res_type(ires_type(ir))(1:5).eq.'L-pep'.or.
     &            res_type(ires_type(ir))(1:5).eq.'D-pep') then
                  if(side_or_main.eq.'S') then
                     number_gr(3,1)       = number_gr(3,1) + 1
                     number_gr(3,ich+1)   = number_gr(3,ich+1) + 1
                     bstats_gr(3,1,1)     = bstats_gr(3,1,1) + bloc
                     bstats_gr(3,1,ich+1) = bstats_gr(3,1,ich+1) + bloc
                     bstats_gr(3,2,1)     = bstats_gr(3,2,1) + bloc2
                     bstats_gr(3,2,ich+1) = bstats_gr(3,2,ich+1) + bloc2
                  else
                     number_gr(2,1)       = number_gr(2,1) + 1
                     number_gr(2,ich+1)   = number_gr(2,ich+1) + 1
                     bstats_gr(2,1,1)     = bstats_gr(2,1,1) + bloc
                     bstats_gr(2,1,ich+1) = bstats_gr(2,1,ich+1) + bloc
                     bstats_gr(2,2,1)     = bstats_gr(2,2,1) + bloc2
                     bstats_gr(2,2,ich+1) = bstats_gr(2,2,ich+1) + bloc2
                  endif
               endif
               number_gr(1,1)       = number_gr(1,1) + 1
               number_gr(1,ich+1)   = number_gr(1,ich+1) + 1
               bstats_gr(1,1,1)     = bstats_gr(1,1,1) + bloc
               bstats_gr(1,1,ich+1) = bstats_gr(1,1,ich+1) + bloc
               bstats_gr(1,2,1)     = bstats_gr(1,2,1) + bloc2
               bstats_gr(1,2,ich+1) = bstats_gr(1,2,ich+1) + bloc2
            endif
         endif
      enddo
c
      do i=1,3
         do l=1,n_group+1
            if(number_gr(i,l).gt.0) then
               bstats_gr(i,1,l) = bstats_gr(i,1,l)/number_gr(i,l)
               bstats_gr(i,2,l) = 
     &              sqrt(max(0.0,bstats_gr(i,2,l)/number_gr(i,l)-
     &                    bstats_gr(i,1,l)**2))
            endif
         enddo
      enddo
c
      return
      end
c
      subroutine  calc_occup_stats(nch_this,number_gr,ostats_gr)
      use rharvest
      implicit none
      include 'atom_com.fh'
      include 'models.fh'
      include 'refi_flags.fh'
c
      integer nch_this
      integer number_gr(3,nch_this)
      real ostats_gr(3,2,nch_this)
c
c---Calculate overall B values for all atoms, main and side chains      
      integer ich
      integer i,j,k,l,ir,nside,nmain,nall
      real occ,occ2
      character side_or_main*1
c
      do i=1,3
         do l=1,nch_this
            number_gr(i,l) = 0
            do k=1,2
               ostats_gr(i,k,l) = 0.0
            enddo
         enddo
      enddo
c
      do i=1,n_atom
         ir = i_resid(i)
         ich = i_chain(ir)
         if(occup_mod(i,1).gt.0.0) then
            if(atom_ref_mod_flag(i,1).gt.0) then
               occ   = occup_mod(i,1)
               occ2  = occ**2
               call check_side_or_main(atm_name(i),side_or_main)
               ir = i_resid(i)
               if(res_type(ires_type(ir))(1:5).eq.'L-pep'.or.
     &            res_type(ires_type(ir))(1:5).eq.'D-pep') then
                  if(side_or_main.eq.'S') then
                     number_gr(3,1)       = number_gr(3,1) + 1
                     number_gr(3,ich+1)   = number_gr(3,ich+1) + 1
                     ostats_gr(3,1,1)     = ostats_gr(3,1,1) + occ
                     ostats_gr(3,1,ich+1) = ostats_gr(3,1,ich+1) + occ
                     ostats_gr(3,2,1)     = ostats_gr(3,2,1) + occ2
                     ostats_gr(3,2,ich+1) = ostats_gr(3,2,ich+1) + occ2
                  else
                     number_gr(2,1)       = number_gr(2,1) + 1
                     number_gr(2,ich+1)   = number_gr(2,ich+1) + 1
                     ostats_gr(2,1,1)     = ostats_gr(2,1,1) + occ
                     ostats_gr(2,1,ich+1) = ostats_gr(2,1,ich+1) + occ
                     ostats_gr(2,2,1)     = ostats_gr(2,2,1) + occ2
                     ostats_gr(2,2,ich+1) = ostats_gr(2,2,ich+1) + occ2
                  endif
               endif
               number_gr(1,1)       = number_gr(1,1) + 1
               number_gr(1,ich+1)   = number_gr(1,ich+1) + 1
               ostats_gr(1,1,1)     = ostats_gr(1,1,1) + occ
               ostats_gr(1,1,ich+1) = ostats_gr(1,1,ich+1) + occ
               ostats_gr(1,2,1)     = ostats_gr(1,2,1) + occ2
               ostats_gr(1,2,ich+1) = ostats_gr(1,2,ich+1) + occ2
            endif
         endif
      enddo
c
      do i=1,3
         do l=1,n_group+1
            if(number_gr(i,l).gt.0) then
               ostats_gr(i,1,l) = ostats_gr(i,1,l)/number_gr(i,l)
               ostats_gr(i,2,l) = 
     &              sqrt(max(0.0,ostats_gr(i,2,l)/number_gr(i,l)-
     &                    ostats_gr(i,1,l)**2))
            endif
         enddo
      enddo
c
      return
      end
c
      subroutine write_xml(ierr,message)
      use rharvest
      use solvent_all
      implicit none
      include 'atom_com.fh'
      include 'refi_flags.fh'
      include 'twin_refmac.fh'
c
      character message*(*)
      integer ierr
c
      character local_file*512
      character file_pdb*512,file_mtz*512
      integer l
c
c---  xml things
      character xinfo*128,xtag*32,xattr*32
      character xtag_a(6)*32,xtag_a1(3)*32
      character xtag_glob*32,xtag_g0*32,xtag_g1*32,xtag_g2*32
      character xtag_g3*32,xtag_g4*32
      character xval*512,xattr_val*512
c
c---
      integer ierr_l
      character line_xml*512
      integer i,j,k,nfirst_geom
      integer ich
      integer in,ifail,ll
c
c---  twin things
      integer itw,itw1
      real tw_max
      real rot_l(3,3),tr_l(3)
c
c---For stats
      integer nch_this
      integer, allocatable :: number_gr(:,:)
      real,    allocatable :: bstats_gr(:,:,:)
      real,    allocatable :: ostats_gr(:,:,:)
c
c---  body
      call getenv('XMLOUT',local_file)
      if(local_file.eq.' ') return
      call open_form_file(in,local_file,ierr_l)
      if(ierr_l.gt.0) then
         write(*,*)'Error in opening xml file'
         return
      endif
c
c--write all necessary info
      l = len_trim(message)
      call xml_version(in)
      call xml_comment(in,'refmac output')
      xinfo = ' '
      xtag_glob = 'REFMAC'
      xattr = ' '
      xattr_val = ' '
      call xml_opentag(in,xinfo,xtag_glob,xattr,xattr_val)
      xinfo = 'Error'
      xtag = 'err_level'
      write(xval,'(i10)')ierr
      call xml_oneliner(in,xinfo,xtag,xval,xattr,xattr_val)
      xinfo = 'Message'
      xtag = 'err_message'
      xval = message
      call xml_oneliner(in,xinfo,xtag,xval,xattr,xattr_val)
      xinfo = 'Job'
      xtag = 'job'
      xval = 'refmac'
      call xml_oneliner(in,xinfo,xtag,xval,xattr,xattr_val)
      if(ierr.gt.0) then
         call xml_closetag(in,xtag_glob)
         close(in)
         return
      endif
c
      xinfo = ' '
      xtag_g0 = 'Overall_stats'
      call xml_opentag(in,xinfo,xtag_g0,xattr,xattr_val)
         
      xtag  = 'n_cycle'
      write(xval,'(i6)')ncycle_overall
      xattr_val = ' '
      call xml_oneliner(in,xinfo,xtag,xval,xattr,xattr_val)
      xinfo = ' '
      xtag  = 'ml_esu'
      write(xval,'(f10.4)')hesu_ml
      call xml_oneliner(in,xinfo,xtag,xval,xattr,xattr_val)

      xtag = 'n_reflections_all'
      write(xval,'(i12)')nhrefl_free+nhrefl_work
      call xml_oneliner(in,xinfo,xtag,xval,xattr,xattr_val)

      xtag = 'n_reflections_free'
      write(xval,'(i12)')nhrefl_free
      call xml_oneliner(in,xinfo,xtag,xval,xattr,xattr_val)

      xtag = 'data_completeness'
      write(xval,'(f10.3)')hperc_shell_refl(hnshell+1)
      call xml_oneliner(in,xinfo,xtag,xval,xattr,xattr_val)

      xtag = 'resolution_low'
      write(xval,'(f10.3)')hd_low
      call xml_oneliner(in,xinfo,xtag,xval,xattr,xattr_val)

      xtag = 'resolution_high'
      write(xval,'(f10.3)')hd_high
      call xml_oneliner(in,xinfo,xtag,xval,xattr,xattr_val)

      xinfo = ' '
      xtag_g1 = 'stats_vs_cycle'
      call xml_opentag(in,xinfo,xtag_g1,xattr,xattr_val)

      line_xml = ' '
      do i=1,ncycle_overall
         xinfo = ' '
         xtag_g2 = 'new_cycle'
         call xml_opentag(in,xinfo,xtag_g2,xattr,xattr_val)
         xtag  = 'cycle'
         write(xval,'(i7)')i-1
         call xml_additem(in,line_xml,xinfo,xtag,xval,xattr,xattr_val)
         xtag  = 'r_factor'
         write(xval,'(f10.3)')rfactor_vs_cycle(i)
         call xml_additem(in,line_xml,xinfo,xtag,xval,xattr,xattr_val)
         xtag = 'r_free'
         write(xval,'(f10.3)')rfree_vs_cycle(i)
         call xml_additem(in,line_xml,xinfo,xtag,xval,xattr,xattr_val)
         xtag  = 'rmsBOND'         
         write(xval,'(f10.4)')bond_vs_cycle(i)
         call xml_additem(in,line_xml,xinfo,xtag,xval,xattr,xattr_val)
         xtag = 'rmsANGLE'
         write(xval,'(f10.3)')angle_vs_cycle(i)
         call xml_additem(in,line_xml,xinfo,xtag,xval,xattr,xattr_val)
         xtag  = 'rmsCHIRAL'
         write(xval,'(f10.3)')chir_vs_cycle(i)
         call xml_additem(in,line_xml,xinfo,xtag,xval,xattr,xattr_val)
         
         call xml_dump(in,line_xml)
         call xml_closetag(in,xtag_g2)
      enddo
      call xml_closetag(in,xtag_g1)
c---
c---  Careful here. Case TLS should be considered
      
c
c---Add some info about geometry also

c
c---  B values
      xtag_a1(1) = 'all'
      xtag_a1(2) = 'main_chain'
      xtag_a1(3) = 'side_chain'
      if(bref_type.ne.'OVER') then
         xtag_g1 = 'bvalue_stats'
         call xml_opentag(in,xinfo,xtag_g1,xattr,xattr_val)
c
c---  Calculate statistics
         allocate(number_gr(3,n_group+1))
         allocate(bstats_gr(3,2,n_group+1))
         nch_this = n_group+1
         call calc_bvalue_stats(nch_this,number_gr,bstats_gr)         
         
         xtag_g2 = 'overall'
         call xml_opentag(in,xinfo,xtag_g2,xattr,xattr_val)

         line_xml = ' '
         do i=1,3
            xtag_g4 = xtag_a1(i)
            call xml_opentag(in,xinfo,xtag_g4,xattr,xattr_val)
            xtag = 'number'
            write(xval,'(i10)')number_gr(i,1)
            call xml_additem(in,line_xml,xinfo,xtag,xval,
     &           xattr,xattr_val)
            xtag = 'average'
            write(xval,'(f10.4)')bstats_gr(i,1,1)
            call xml_additem(in,line_xml,xinfo,xtag,xval,
     &           xattr,xattr_val)
            xtag = 'sigma'
            write(xval,'(f10.4)')bstats_gr(i,2,1)
            call xml_additem(in,line_xml,xinfo,xtag,xval,
     &           xattr,xattr_val)
            call xml_dump(in,line_xml)
            call xml_closetag(in,xtag_g4)
         enddo
         call xml_closetag(in,xtag_g2)

         if(n_group.gt.0) then
            xtag_g2 = 'chain_by_chain'
            call xml_opentag(in,xinfo,xtag_g2,xattr,xattr_val)
            xinfo = ' '
            do l=1,n_group
               xtag_g3 = 'new_chain'
               call xml_opentag(in,xinfo,xtag_g3,xattr,xattr_val)
               xtag = 'chain_name'
               xval = group_id(l)
               call xml_oneliner(in,xinfo,xtag,xval,xattr,xattr_val)
               do i=1,3
                  xtag_g4 = xtag_a1(i)
                  call xml_opentag(in,xinfo,xtag_g4,xattr,xattr_val)
                  xtag = 'number'
                  write(xval,'(i10)')number_gr(i,l+1)
                  call xml_additem(in,line_xml,xinfo,xtag,xval,
     &                 xattr,xattr_val)
                  xtag = 'average'
                  write(xval,'(f10.4)')bstats_gr(i,1,l+1)
                  call xml_additem(in,line_xml,xinfo,xtag,xval,
     &                 xattr,xattr_val)
                  xtag = 'sigma'
                  write(xval,'(f10.4)')bstats_gr(i,2,l+1)
                  call xml_additem(in,line_xml,xinfo,xtag,xval,
     &                 xattr,xattr_val)
                  call xml_dump(in,line_xml)
                  call xml_closetag(in,xtag_g4)
               enddo
               call xml_closetag(in,xtag_g3)
            enddo
            call xml_closetag(in,xtag_g2)
         endif
         call xml_closetag(in,xtag_g1)
         deallocate(number_gr)
         deallocate(bstats_gr)
      endif
c
c--   Ae we refining occupancies

      if(mode.eq.'OCCU') then
c
c---  Calculate statistics
         xtag_g1 = 'occupancy_stats'
         call xml_opentag(in,xinfo,xtag_g1,xattr,xattr_val)

         allocate(number_gr(3,n_group+1))
         allocate(ostats_gr(3,2,n_group+1))
         nch_this = n_group+1
         call calc_occup_stats(nch_this,number_gr,ostats_gr)         
         xtag_g2 = 'overall'
         call xml_opentag(in,xinfo,xtag_g2,xattr,xattr_val)

         line_xml = ' '
         do i=1,3
            xtag_g4 = xtag_a1(i)
            call xml_opentag(in,xinfo,xtag_g4,xattr,xattr_val)
            xtag = 'number'
            write(xval,'(i10)')number_gr(i,1)
            call xml_additem(in,line_xml,xinfo,xtag,xval,
     &           xattr,xattr_val)
            xtag = 'average'
            write(xval,'(f10.4)')ostats_gr(i,1,1)
            call xml_additem(in,line_xml,xinfo,xtag,xval,
     &           xattr,xattr_val)
            xtag = 'sigma'
            write(xval,'(f10.4)')ostats_gr(i,2,1)
            call xml_additem(in,line_xml,xinfo,xtag,xval,
     &           xattr,xattr_val)
            call xml_dump(in,line_xml)
            call xml_closetag(in,xtag_g4)
         enddo
         call xml_closetag(in,xtag_g2)

         if(n_group.gt.0) then
            xinfo = 'Chain bvalue stats'
            xtag_g2 = 'chain_by_chain'
            call xml_opentag(in,xinfo,xtag_g2,xattr,xattr_val)
            xinfo = ' '
            do l=1,n_group
               xtag_g3 = 'new_chain'
               call xml_opentag(in,xinfo,xtag_g3,xattr,xattr_val)
               xtag = 'chain_name'
               xval = group_id(l)
               call xml_oneliner(in,xinfo,xtag,xval,xattr,xattr_val)
               do i=1,3
                  xtag_g4 = xtag_a1(i)
                  call xml_opentag(in,xinfo,xtag_g4,xattr,xattr_val)
                  xtag = 'number'
                  write(xval,'(i10)')number_gr(i,l+1)
                  call xml_additem(in,line_xml,xinfo,xtag,xval,
     &                 xattr,xattr_val)
                  xtag = 'average'
                  write(xval,'(f10.4)')ostats_gr(i,1,l+1)
                  call xml_additem(in,line_xml,xinfo,xtag,xval,
     &                 xattr,xattr_val)
                  xtag = 'sigma'
                  write(xval,'(f10.4)')ostats_gr(i,2,l+1)
                  call xml_additem(in,line_xml,xinfo,xtag,xval,
     &                 xattr,xattr_val)
                  call xml_dump(in,line_xml)
                  call xml_closetag(in,xtag_g4)
               enddo
               call xml_closetag(in,xtag_g3)
            enddo
            call xml_closetag(in,xtag_g2)
         endif
         call xml_closetag(in,xtag_g1)

         deallocate(number_gr)
         deallocate(ostats_gr)
      endif
      call xml_closetag(in,xtag_g0)
c
c---Any other info

c
c---  Solvent parameters
      xinfo = ' '
      xtag_g1 = 'solvent_info'
      call xml_opentag(in,xinfo,xtag_g1,xattr,xattr_val)
      xtag_g2 = 'params_rfree'

      call xml_opentag(in,xinfo,xtag_g2,xattr,xattr_val)
      xtag='vdw_probe_radius'
      write(xval,'(f10.4)')prob_vdw_rfree
      call xml_additem(in,line_xml,xinfo,xtag,xval,xattr,
     &           xattr_val)
      xtag = 'ion_probe_radius'
      write(xval,'(f10.4)')prob_ion_rfree
      call xml_additem(in,line_xml,xinfo,xtag,xval,xattr,
     &           xattr_val)
      xtag = 'shrinkage_radius'
      write(xval,'(f10.4)')radii_shrink_rfree
      call xml_additem(in,line_xml,xinfo,xtag,xval,xattr,
     &           xattr_val)
      call xml_dump(in,line_xml)
      call xml_closetag(in,xtag_g2)

      xtag_g2 = 'params_rfactor'

      call xml_opentag(in,xinfo,xtag_g2,xattr,xattr_val)
      xtag='vdw_probe_radius'
      write(xval,'(f10.4)')prob_vdw_rfactor
      call xml_additem(in,line_xml,xinfo,xtag,xval,xattr,
     &           xattr_val)
      xtag = 'ion_probe_radius'
      write(xval,'(f10.4)')prob_ion_rfactor
      call xml_additem(in,line_xml,xinfo,xtag,xval,xattr,
     &           xattr_val)
      xtag = 'shrinkage_radius'
      write(xval,'(f10.4)')radii_shrink_rfactor
      call xml_additem(in,line_xml,xinfo,xtag,xval,xattr,
     &           xattr_val)
      call xml_dump(in,line_xml)
      call xml_closetag(in,xtag_g2)
      call xml_closetag(in,xtag_g1)

c
c---Twin
      xinfo = ' '
      xtag_g1 = 'twin_info'
      call xml_opentag(in,xinfo,xtag_g1,xattr,xattr_val) 
      xtag = 'ntwin_domain'
      write(xval,'(i5)')ntwin_domain
      call xml_oneliner(in,xinfo,xtag,xval,xattr,xattr_val)
      if(twin_flag.and.ntwin_domain.gt.1) then
         do i=1,ntwin_domain
            xinfo = ' '
            xtag_g2 = 'new_domain'
            call xml_opentag(in,xinfo,xtag_g2,xattr,xattr_val)
            xtag = 'domain'
            write(xval,'(i5)')i
            call xml_additem(in,line_xml,xinfo,xtag,xval,xattr,
     &           xattr_val)
            xtag = 'operator'
            rot_l(1:3,1:3) = float(twin_oper(1:3,1:3,i))/12.0
            tr_l(1:3) = 0.0
            call put_symm_to_text(rot_l,tr_l,xval)
            call xml_additem(in,line_xml,xinfo,xtag,xval,xattr,
     &           xattr_val)
            xtag = 'fraction'
            write(xval,'(f5.3)')twin_frac(i)
            call xml_additem(in,line_xml,xinfo,xtag,xval,xattr,
     &           xattr_val)
            call xml_dump(in,line_xml)
            call xml_closetag(in,xtag_g2)
         enddo
         xtag_g2 = 'reindex'
         call xml_opentag(in,xinfo,xtag_g2,xattr,xattr_val)
         if(twin_frac(1)*ntwin_domain.lt.0.97) then
            write(*,*)'Before looping over'
            tw_max = -1.0e32
            do itw=1,ntwin_domain
               if(tw_max.lt.twin_frac(itw)) then
                  itw1 = itw
                  tw_max = twin_frac(itw)
               endif
            enddo
            
            rot_l(1:3,1:3) =twin_oper(1:3,1:3,itw1)/12.0
            tr_l(1:3) = 0.0
            call put_symm_to_text(rot_l,tr_l,xval)
            xtag = 'action'
            xval = 'reindex'
            call getenv('HKLIN',file_mtz)
            call getenv('XYZIN',file_pdb)
            xval = 'pointless HKLIN '//trim(file_mtz)
            xval = trim(xval)//' XYZIN '//trim(file_pdb)
            xval = trim(xval)//' HKLOUT '//
     &           trim(file_mtz)//'.pointless.mtz'
            call xml_oneliner(in,xinfo,xtag,xval,xattr,xattr_val)
            xtag = 'operator'
            call put_symm_to_text(rot_l,tr_l,xval)
            call xml_oneliner(in,xinfo,xtag,xval,xattr,xattr_val)
         else
            xtag = 'action'
            xval = 'none'
            call xml_oneliner(in,xinfo,xtag,xval,xattr,xattr_val)
            xtag = 'operator'
            xval = 'NULL'
            call xml_oneliner(in,xinfo,xtag,xval,xattr,xattr_val)
         endif
         call xml_closetag(in,xtag_g2)       
      endif
      call xml_closetag(in,xtag_g1)
c
c---  end of twin
      xinfo = ' '
      xval  = ' '
      call getenv('XYZOUT',file_pdb)
      if(file_pdb.eq.' ') file_pdb = 'XYZOUT'
      xinfo = ' '
      xtag = 'solution_file'
      xattr = ' '
      xval = file_pdb
      call xml_oneliner(in,xinfo,xtag,xval,xattr,xattr_val)

      call getenv('HKLOUT',file_mtz)
      if(file_mtz.eq.' ') file_mtz = 'HKLOUT'
      xtag = 'mtz_file'
      xattr = ' '
      xval = file_mtz
      call xml_oneliner(in,xinfo,xtag,xval,xattr,xattr_val)

      call xml_closetag(in,xtag_glob)
      close(in)

      return
      end
c
      subroutine check_end
      implicit none
c     

      return
      end
